/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 1
 * Digital I/O
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>     /* srand, rand */

/*
 In PORTA vom avea primele 8 leduri de joc;
 In PORTC vom avea primele 8 butoane de joc;
 
 Ledul ramas de joc e pe PB0, butonul ramas de joc e pe PB1;
 Ledul verde care afiseaza dificultatea e pe PB2;
 Ledul albastru care afiseaza dificultatea e pe PB3;
 
 Buton de start game e pe PD4;
 Buton de schimbat dificultatea e pe PD5;
 */

// Aprindem doar ledul 0
void on_led0()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA0);
	PORTA &= ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 1
void on_led1()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA1);
	PORTA &= ~(1 << PA0) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 2
void on_led2()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA2);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 3
void on_led3()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA3);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 4
void on_led4()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA4);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 5
void on_led5()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA5);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA6) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 6
void on_led6()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA6);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA7);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 7
void on_led7()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PA7);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6);
	PORTB &= ~(1 << PB0);
}

// Aprindem doar ledul 8
void on_led8()
{
	DDRA = 0xff;
	DDRB |= (1 << PB0);
	
	PORTA |= (1 << PB0);
	PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
}

void on_led(int nr)
{
	switch (nr)
	{
		case 0:
			on_led0();
			break;
		case 1:
			on_led1();
			break;
		case 2:
			on_led2();
			break;
		case 3:
			on_led3();
			break;
		case 4:
			on_led4();
			break;
		case 5:
			on_led5();
			break;
		case 6:
			on_led6();
			break;
		case 7:
			on_led7();
			break;
		case 8:
			on_led8();
			break;
	}
}

int main()
{

	int game_diff = 0;
	int start_game = 0;
    int randomize = 0;
	
	// Aprinem ledul verde de schimbare dificultate ca initializare
	DDRA = 0xff;
	DDRB |= (1 << PB2);
	PORTB |= (1 << PB2);
	_delay_ms(100);
	
	// Punem pinul PD4 ca input si activam rezistenta de pull-up asociata pinului respectiv
	// PD4 = buton de start game
	DDRD &= ~(1 << PD4);
	PORTD |= (1 << PD4);
	_delay_ms(100);
	
	// Punem pinul PD5 ca input si activam rezistenta de pull-up asociata pinului respectiv
	//PD5 = buton de schimbare dificultate
	DDRD &= ~(1 << PD5);
	PORTD |= (1 << PD5);
	_delay_ms(100);
	
	while(1)
	{		
  
        randomize += 3;
		
		// Calculez de cate ori am apasat pe butonul de schimbare dificultate
		if (! (PIND & (1 << PD5)))
		{
			game_diff++;
		}
		_delay_ms(100);
		
		// Calculez de cate ori am apasat pe butonul de start game
		if (! (PIND & (1 << PD4)))
		{
			start_game++;
		}
        
        srand (randomize);
        
        if (start_game % 2 == 0)
        {
            //reset game
			PORTA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
			PORTB &= ~(1 << PB0);
        } else {
            //start game, incep sa aprind ledurile random
            on_led(rand() % 9);
            _delay_ms(10);
        }

		
		if (game_diff % 2 == 1)
		{
			// daca am apasat de un numar impar de ori, atunci am schimbat dificultatea ridicata (led albastru)
			PORTB |= (1 << PB3);
			PORTB &= ~(1 << PB2);
			_delay_ms(10);
		} else {
			// altfel, am apasat de un numar par de ori, deci am schimbat pe dificultatea usoara (led verde)
			if (game_diff % 2 == 0)
			{
				PORTB |= (1 << PB2);
				PORTB &= ~(1 << PB3);
				_delay_ms(10);
			}
		}
		
		// Functie care verifica daca am apasat pe butonul corect.
		
	}
	
	return 0;
}
